/***************************************************************************
                          mygeomtransformdialog.cpp  -  description
                             -------------------
    begin                : Fri Jul 6 2001
    copyright            : (C) 2001 by Rdiger Knrig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qtooltip.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qgroupbox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qmessagebox.h>

#include "mygeomtransformdialog.h"
#include "my3dcoordentry.h"

MygeomTransformDialog::MygeomTransformDialog(QWidget *parent, const char *name ) : QDialog(parent,name,true)
{
	setCaption(tr("geometric transforms"));
	char coordstr[] = "x";
	
	mainlayout = new QVBoxLayout(this,10,10,"mainlayout");
	centrallayout = new QGridLayout(mainlayout,2,2,5);
	centrallayout->setColStretch(1,2);
	centrallayout->setColStretch(0,0);
	buttonlayout = new QHBoxLayout(mainlayout);
	
	transformlistview = new QListView(this);
	transformlistview->setSorting(-1,true); // disable sorting
	transformlistview->addColumn(tr("transformation"),200);
	for(int i=1; i <= 3; i++)
	{
		transformlistview->addColumn(coordstr,100);
		transformlistview->setColumnAlignment(i,Qt::AlignHCenter);
		coordstr[0]++;
	}
	centrallayout->addWidget(transformlistview,0,1);
	
	listopgroupbox = new QGroupBox(1,Qt::Horizontal,tr("list operations"),this);
	appendbutton = new QPushButton("&Append",listopgroupbox);
	QToolTip::add(appendbutton,tr("append to end of list"));
	insertbutton = new QPushButton("Insert",listopgroupbox);
	QToolTip::add(insertbutton,tr("insert after current element"));
	alterbutton = new QPushButton("A&lter",listopgroupbox);
	QToolTip::add(alterbutton,tr("change current element"));
	deletebutton = new QPushButton("Delete",listopgroupbox);
	QToolTip::add(deletebutton,tr("delete current element"));
	upbutton = new QToolButton(UpArrow,listopgroupbox);
	QToolTip::add(upbutton,tr("moves current element up in list"));
	downbutton = new QToolButton(DownArrow,listopgroupbox);
	QToolTip::add(downbutton,tr("moves current element down in list"));
	centrallayout->addWidget(listopgroupbox,0,0);
	
	geomopgroupbox = new QGroupBox(3,Qt::Horizontal,tr("geometric operations"),this);
	new QLabel("operation type:   \t",geomopgroupbox);
	transformscombo = new QComboBox(geomopgroupbox);
	transformscombo->insertItem("translate");
	transformscombo->insertItem("scale");
	transformscombo->insertItem("rotate");
	geomopgroupbox->addSpace(0);
	
	new QLabel("operation argument:\t",geomopgroupbox);
	transformargentry = new My3DCoordEntry(geomopgroupbox);
	centrallayout->addMultiCellWidget(geomopgroupbox,1,1,0,1);
	
	okbutton = new QPushButton("&Ok",this);
	QToolTip::add(okbutton,tr("accept transform"));
	cancelbutton = new QPushButton("Cancel",this);
	cancelbutton->setAccel(Key_Escape);
	QToolTip::add(cancelbutton,tr("forget it..."));
	geomopgroupbox->addSpace(0);
	
	buttonlayout->addWidget(okbutton,0,Qt::AlignBottom | Qt::AlignLeft);
	buttonlayout->addStretch(2);
	buttonlayout->addWidget(cancelbutton,0,Qt::AlignBottom | Qt::AlignRight);
	
	connect(okbutton,SIGNAL(clicked()),this,SLOT(accept()));
	connect(cancelbutton,SIGNAL(clicked()),this,SLOT(reject()));
	connect(appendbutton,SIGNAL(clicked()),this,SLOT(slotAppend()));
	connect(insertbutton,SIGNAL(clicked()),this,SLOT(slotInsert()));
	connect(deletebutton,SIGNAL(clicked()),this,SLOT(slotDelete()));
	connect(alterbutton,SIGNAL(clicked()),this,SLOT(slotAlter()));
	connect(upbutton,SIGNAL(clicked()),this,SLOT(slotMoveUp()));
	connect(downbutton,SIGNAL(clicked()),this,SLOT(slotMoveDown()));
	connect(transformlistview,SIGNAL(clicked(QListViewItem *)),this,SLOT(slotCpyToEntry(QListViewItem *)));
	
}

MygeomTransformDialog::~MygeomTransformDialog()
{
}

MyPoly3D::tmatrix MygeomTransformDialog::getTransform()
{
	QString transformname;
	float value;
	// calculate transformation matrix;
	transfmatrix.reset();
	
	QListViewItemIterator it(transformlistview);
	for ( ; it.current(); ++it )
	{
		transformname = it.current()->text(0);
		if(transformname == "translate") transfmatrix.translate(it.current()->text(1).toFloat(),it.current()->text(2).toFloat(),it.current()->text(3).toFloat());
		if(transformname == "scale") transfmatrix.scale(it.current()->text(1).toFloat(),it.current()->text(2).toFloat(),it.current()->text(3).toFloat());
		if(transformname == "rotate")
			{
				if((value = it.current()->text(1).toFloat()) != 0.0) transfmatrix.rotateAroundX(value);
				if((value = it.current()->text(2).toFloat()) != 0.0) transfmatrix.rotateAroundY(value);
				if((value = it.current()->text(3).toFloat()) != 0.0) transfmatrix.rotateAroundZ(value);
			}
	}
	
	return(transfmatrix);
}

/**
	* append a transform to the end of the
	* list
	*/
	
void MygeomTransformDialog::slotAppend()
{
	if(transform_t(transformscombo->currentItem()) == scale)
		{
			if(transformargentry->getX() == 0.0 || transformargentry->getY() == 0.0 || transformargentry->getZ() == 0.0)
			{
				QMessageBox::information(this,tr("attention!"),tr("You specified a scaling operation, but some\n"
				"scaling factors have zero values!"));
				return;
			}
		}
	QString temp;
	QListViewItem *newitem = new QListViewItem(transformlistview,transformscombo->currentText(),\
	temp.sprintf("%f",transformargentry->getX()),\
	temp.sprintf("%f",transformargentry->getY()),\
	temp.sprintf("%f",transformargentry->getZ()));
	transformlistview->insertItem(newitem);
	QListViewItem *tempitem = transformlistview->firstChild();
	for(;tempitem->itemBelow() != 0; tempitem = tempitem->itemBelow());
	newitem->moveItem(tempitem);
	
}

/** deletes the selected transform */
void MygeomTransformDialog::slotDelete()
{
	QListViewItem *item = transformlistview->selectedItem();
	
	if(item != 0)
		{
			transformlistview->takeItem(item);
			delete item;
		}
}


/**
	* copies transformation attributes from list
	* to the edit entry if the user selects them
	* per mouse click
	*/
void MygeomTransformDialog::slotCpyToEntry(QListViewItem *item)
{
	if(item != 0)
	{
		QString transformname = item->text(0);
		if(transformname == "translate") transformscombo->setCurrentItem((int) translate);
		if(transformname == "scale") transformscombo->setCurrentItem((int) scale);
		if(transformname == "rotate") transformscombo->setCurrentItem((int) rotate);
		transformargentry->setX(item->text(1).toFloat());
		transformargentry->setY(item->text(2).toFloat());
		transformargentry->setZ(item->text(3).toFloat());
	}
}

/**
	* insert the transform after the selected
	* transform
	*/
void MygeomTransformDialog::slotInsert()
{
	QListViewItem *item = transformlistview->selectedItem();
	
	int childs = transformlistview->childCount();
	slotAppend();
	
	if(item != 0 && childs < transformlistview->childCount())
	{
		transformlistview->firstChild()->moveItem(item);
	}
	
}

/** moves current item up if possible */
void MygeomTransformDialog::slotMoveUp()
{
	QListViewItem *item = transformlistview->selectedItem();
	
	if(item != 0 && item->itemAbove() != 0)
	{
		if(item->itemAbove()->itemAbove() == 0)
			{
				// move to the first pos
				transformlistview->takeItem(item);
				transformlistview->insertItem(item);
				transformlistview->setSelected(item,true);
			}
			else
			{
				item->moveItem(item->itemAbove()->itemAbove());
			}
	}
	transformlistview->repaint();
}

/** moves current item down if possible */
void MygeomTransformDialog::slotMoveDown()
{
	QListViewItem *item = transformlistview->selectedItem();
	
	if(item != 0 && item->itemBelow() != 0)
	{
		item->moveItem(item->itemBelow());
	}
	transformlistview->repaint();
}

/**
	* current selections becomes values
	* from the property field
	*/
void MygeomTransformDialog::slotAlter()
{
	QListViewItem *item = transformlistview->selectedItem();
	
	if(item != 0)
	{
		if(transform_t(transformscombo->currentItem()) == scale)
		{
			if(transformargentry->getX() == 0.0 || transformargentry->getY() == 0.0 || transformargentry->getZ() == 0.0)
			{
				QMessageBox::information(this,tr("attention!"),tr("You specified a scaling operation, but some\n"
				"scaling factors have zero values!"));
				return;
			}
		}
	QString temp;
	item->setText(0,transformscombo->currentText());
	item->setText(1,temp.sprintf("%f",transformargentry->getX()));
	item->setText(2,temp.sprintf("%f",transformargentry->getY()));
	item->setText(3,temp.sprintf("%f",transformargentry->getZ()));
	
	}
}
