/***************************************************************************
                          mygeomtransformdialog.h  -  description
                             -------------------
    begin                : Fri Jul 6 2001
    copyright            : (C) 2001 by Rdiger Knrig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MYGEOMTRANSFORMDIALOG_H
#define MYGEOMTRANSFORMDIALOG_H

// qt specific includes

#include <qwidget.h>
#include <qdialog.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qlistview.h>

// class specific includes
#include "mypoly3d.h"

class QPushButton;
class QToolButton;
class QGroupBox;
class QComboBox;
class My3DCoordEntry;

/**
	* Implementation of a dialog letting the
	* user describe geometric transforms
	* to the scene
  * @author Rdiger Knrig
  */

class MygeomTransformDialog : public QDialog  {
   Q_OBJECT
public:
	/** default constructor */
	MygeomTransformDialog(QWidget *parent=0, const char *name=0);
	/** default destructor */
	~MygeomTransformDialog();
	/** getter for the transformation matrix */
	MyPoly3D::tmatrix getTransform();
protected:
	/** possible geometric operations */
	enum transform_t {translate,scale,rotate};
	/** main layout */
	QVBoxLayout *mainlayout;
	/** layout for the central part */
	QGridLayout *centrallayout;
	/** layout for the buttons */
	QHBoxLayout *buttonlayout;
	/** listview for the transformations */
	QListView *transformlistview;
	/** groupbox for the list operation buttons */
	QGroupBox *listopgroupbox;
	/** Pushbutton for the append operation */
	QPushButton *appendbutton;
	/** Pushbutton for the insert operation */
	QPushButton *insertbutton;
	/** Pushbutton for the "alter" operation */
	QPushButton *alterbutton;
	/** Pushbutton for the delete operation */
	QPushButton *deletebutton;
	/** Toolbutton for moving element up in the list */
	QToolButton *upbutton;
	/** Toolbutton for moving element down in the list */
	QToolButton *downbutton;
	/** groupbox for elements to specify the geometric operation */
	QGroupBox *geomopgroupbox;
	/** combo for selecting the geometric transformation */
	QComboBox *transformscombo;
	/** 3d coordinate entry for the transform argument */
	My3DCoordEntry *transformargentry;
	/** Ok-Button */
	QPushButton *okbutton;
	/** Cancel-Button */
	QPushButton *cancelbutton;
	/** transformation matrix */
	MyPoly3D::tmatrix transfmatrix;
protected slots: // Protected slots
  /** append a transform to the end of the 
list */
  void slotAppend();
  /** deletes the selected transform */
  void slotDelete();
  /**
  	* copies transformation attributes from list
		* to the edit entry if the user selects them
		* per mouse click */
  void slotCpyToEntry(QListViewItem *);
  /**
  	* insert the transform after the selected
		* transform
		*/
  void slotInsert();
  /** moves current item up if possible */
  void slotMoveUp();
  /** moves current item down if possible */
  void slotMoveDown();
  /** current selections becomes values 
from the property field */
  void slotAlter();
};

#endif
