/***************************************************************************
                          qtplgviewer.cpp  -  description
                             -------------------
    begin                : Don Jun 28 10:55:59 CEST 2001
    copyright            : (C) 2001 by Rdiger Knrig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Qt includes
#include <qvbox.h>
#include <qaccel.h>
#include <qtoolbutton.h>

// application specific includes

#include "qtplgviewerdoc.h"
#include "qtplgviewer.h"
#include "mypoly3d.h"
#include "mygeomtransformdialog.h"
#include "myviewpropertyclass.h"
#include <math.h>

// pixmaps for the file menu
#include "filenew.xpm"
#include "fileopen.xpm"
#include "filesave.xpm"

// pixmaps for the toolbar
#include "orthoxy.xpm"
#include "orthoxz.xpm"
#include "orthoyz.xpm"
#include "persp1.xpm"
#include "wireframe.xpm"
#include "backfaceculling.xpm"
#include "surfaceandwf.xpm"
#include "surface.xpm"

QtpLgviewerApp::QtpLgviewerApp()
{
  setCaption(tr("QtpLgviewer " VERSION ) );

  printer = new QPrinter;
  untitledCount=0;
  pDocList = new QList<QtpLgviewerDoc>();
  pDocList->setAutoDelete(true);

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initView();
  initActions();
  initMenuBar();
  initToolBar();
  initStatusBar();
  resize( 450, 400 );

  viewToolBar->setOn(true);
  viewStatusBar->setOn(true);
  viewViewBar->setOn(true);

}

QtpLgviewerApp::~QtpLgviewerApp()
{
  delete printer;
  delete[] viewtransformbuttons;
  delete[] viewmodebuttons;
}

void QtpLgviewerApp::initActions()
{
  QPixmap openIcon, saveIcon, newIcon;
  newIcon = QPixmap(filenew);
  openIcon = QPixmap(fileopen);
  saveIcon = QPixmap(filesave);

  fileNew = new QAction(tr("New File"), newIcon, tr("&New"), QAccel::stringToKey(tr("Ctrl+N")), this);
  fileNew->setStatusTip(tr("Creates a new document"));
  fileNew->setWhatsThis(tr("New File\n\nCreates a new document"));
  connect(fileNew, SIGNAL(activated()), this, SLOT(slotFileNew()));

  fileOpen = new QAction(tr("Open File"), openIcon, tr("&Open..."), 0, this);
  fileOpen->setStatusTip(tr("Opens an existing document"));
  fileOpen->setWhatsThis(tr("Open File\n\nOpens an existing document"));
  connect(fileOpen, SIGNAL(activated()), this, SLOT(slotFileOpen()));

  fileSave = new QAction(tr("Save File"), saveIcon, tr("&Save"), QAccel::stringToKey(tr("Ctrl+S")), this);
  fileSave->setStatusTip(tr("Saves the actual document"));
  fileSave->setWhatsThis(tr("Save File.\n\nSaves the actual document"));
  connect(fileSave, SIGNAL(activated()), this, SLOT(slotFileSave()));

  fileSaveAs = new QAction(tr("Save File As"), tr("Save &as..."), 0, this);
  fileSaveAs->setStatusTip(tr("Saves the actual document under a new filename"));
  fileSaveAs->setWhatsThis(tr("Save As\n\nSaves the actual document under a new filename"));
  connect(fileSaveAs, SIGNAL(activated()), this, SLOT(slotFileSaveAs()));

  fileClose = new QAction(tr("Close File"), tr("&Close"), QAccel::stringToKey(tr("Ctrl+W")), this);
  fileClose->setStatusTip(tr("Closes the actual document"));
  fileClose->setWhatsThis(tr("Close File\n\nCloses the actual document"));
  connect(fileClose, SIGNAL(activated()), this, SLOT(slotFileClose()));

  filePrint = new QAction(tr("Print File"), tr("&Print"), QAccel::stringToKey(tr("Ctrl+P")), this);
  filePrint->setStatusTip(tr("Prints out the actual document"));
  filePrint->setWhatsThis(tr("Print File\n\nPrints out the actual document"));
  connect(filePrint, SIGNAL(activated()), this, SLOT(slotFilePrint()));

  fileQuit = new QAction(tr("Exit"), tr("E&xit"), QAccel::stringToKey(tr("Ctrl+Q")), this);
  fileQuit->setStatusTip(tr("Quits the application"));
  fileQuit->setWhatsThis(tr("Exit\n\nQuits the application"));
  connect(fileQuit, SIGNAL(activated()), this, SLOT(slotFileQuit()));

  editCut = new QAction(tr("Cut"), tr("Cu&t"), QAccel::stringToKey(tr("Ctrl+X")), this);
  editCut->setStatusTip(tr("Cuts the selected section and puts it to the clipboard"));
  editCut->setWhatsThis(tr("Cut\n\nCuts the selected section and puts it to the clipboard"));
  connect(editCut, SIGNAL(activated()), this, SLOT(slotEditCut()));

  editCopy = new QAction(tr("Copy"), tr("&Copy"), QAccel::stringToKey(tr("Ctrl+C")), this);
  editCopy->setStatusTip(tr("Copies the selected section to the clipboard"));
  editCopy->setWhatsThis(tr("Copy\n\nCopies the selected section to the clipboard"));
  connect(editCopy, SIGNAL(activated()), this, SLOT(slotEditCopy()));

  editUndo = new QAction(tr("Undo"), tr("&Undo"), QAccel::stringToKey(tr("Ctrl+Z")), this);
  editUndo->setStatusTip(tr("Reverts the last editing action"));
  editUndo->setWhatsThis(tr("Undo\n\nReverts the last editing action"));
  connect(editUndo, SIGNAL(activated()), this, SLOT(slotEditUndo()));

  editPaste = new QAction(tr("Paste"), tr("&Paste"), QAccel::stringToKey(tr("Ctrl+V")), this);
  editPaste->setStatusTip(tr("Pastes the clipboard contents to actual position"));
  editPaste->setWhatsThis(tr("Paste\n\nPastes the clipboard contents to actual position"));
  connect(editPaste, SIGNAL(activated()), this, SLOT(slotEditPaste()));

	editTransforms = new QAction(tr("Apply Transformation"),tr("&Transform"),QAccel::stringToKey(tr("Ctrl+T")), this);
	editTransforms->setStatusTip(tr("Apply a geometric transform to the whole scene."));
	editTransforms->setWhatsThis(tr("Apply Transforms\n\nApply a geometric transform (translation,scaling,rotation)\nto the whole scene."));
	connect(editTransforms, SIGNAL(activated()), this, SLOT(slotEditTransformations()));
	
	editViewProperties= new QAction(tr("configure current view"),tr("View Setup"),0, this);
	editViewProperties->setStatusTip(tr("Configure the current view."));
	editViewProperties->setWhatsThis(tr("View Setup\n\nSpecify view parameters like background color or illumination."));
	connect(editViewProperties,SIGNAL(activated()), this, SLOT(slotEditViewSettings()));
	
  viewToolBar = new QAction(tr("Toolbar"), tr("Tool&bar"), 0, this, 0, true);
  viewToolBar->setStatusTip(tr("Enables/disables the toolbar"));
  viewToolBar->setWhatsThis(tr("Toolbar\n\nEnables/disables the toolbar"));
  connect(viewToolBar, SIGNAL(toggled(bool)), this, SLOT(slotViewToolBar(bool)));

  viewStatusBar = new QAction(tr("Statusbar"), tr("&Statusbar"), 0, this, 0, true);
  viewStatusBar->setStatusTip(tr("Enables/disables the statusbar"));
  viewStatusBar->setWhatsThis(tr("Statusbar\n\nEnables/disables the statusbar"));
  connect(viewStatusBar, SIGNAL(toggled(bool)), this, SLOT(slotViewStatusBar(bool)));

  viewViewBar = new QAction(tr("View Transformations Bar"), tr("&Viewtransbar"), 0, this, 0, true);
	viewViewBar->setStatusTip(tr("Enables/disables the view transformation bar"));
	viewViewBar->setWhatsThis(tr("View Transformations Bar\n\nEnables/disables the view transformation bar"));
	connect(viewViewBar, SIGNAL(toggled(bool)), this, SLOT(slotViewViewTransformBar(bool)));

  windowNewWindow = new QAction(tr("New Window"), tr("&New Window"), 0, this);
  windowNewWindow->setStatusTip(tr("Opens a new view for the current document"));
  windowNewWindow->setWhatsThis(tr("New Window\n\nOpens a new view for the current document"));
  connect(windowNewWindow, SIGNAL(activated()), this, SLOT(slotWindowNewWindow()));

  windowCascade = new QAction(tr("Cascade"), tr("&Cascade"), 0, this);
  windowCascade->setStatusTip(tr("Cascades all windows"));
  windowCascade->setWhatsThis(tr("Cascade\n\nCascades all windows"));
  connect(windowCascade, SIGNAL(activated()), pWorkspace, SLOT(cascade()));

  windowTile = new QAction(tr("Tile"), tr("&Tile"), 0, this);
  windowTile->setStatusTip(tr("Tiles all windows"));
  windowTile->setWhatsThis(tr("Tile\n\nTiles all windows"));
  connect(windowTile, SIGNAL(activated()), pWorkspace, SLOT(tile()));


  windowAction = new QActionGroup(this, 0, false);
  windowAction->insert(windowNewWindow);
  windowAction->insert(windowCascade);
  windowAction->insert(windowTile);

  helpAboutApp = new QAction(tr("About"), tr("&About..."), 0, this);
  helpAboutApp->setStatusTip(tr("About the application"));
  helpAboutApp->setWhatsThis(tr("About\n\nAbout the application"));
  connect(helpAboutApp, SIGNAL(activated()), this, SLOT(slotHelpAbout()));

  helpListKeyBindings = new QAction(tr("list key bindings"),tr("key bindings.."),0,this);
	helpListKeyBindings->setStatusTip(tr("show key bindings"));
	helpListKeyBindings->setWhatsThis(tr("Key bindings\n\nShow key command for navigate through the scenary."));
	connect(helpListKeyBindings, SIGNAL(activated()), this, SLOT(slotHelpKeys()));
}

void QtpLgviewerApp::initMenuBar()
{
  ///////////////////////////////////////////////////////////////////
  // MENUBAR

  ///////////////////////////////////////////////////////////////////
  // menuBar entry pFileMenu
  pFileMenu=new QPopupMenu();
  fileNew->addTo(pFileMenu);
  fileOpen->addTo(pFileMenu);
  fileClose->addTo(pFileMenu);
  pFileMenu->insertSeparator();
  fileSave->addTo(pFileMenu);
  fileSaveAs->addTo(pFileMenu);
  pFileMenu->insertSeparator();
  filePrint->addTo(pFileMenu);
  pFileMenu->insertSeparator();
  fileQuit->addTo(pFileMenu);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry editMenu
  pEditMenu=new QPopupMenu();
  //editUndo->addTo(pEditMenu);
  //pEditMenu->insertSeparator();
  //editCut->addTo(pEditMenu);
  //editCopy->addTo(pEditMenu);
  //editPaste->addTo(pEditMenu);
	editTransforms->addTo(pEditMenu);
	editViewProperties->addTo(pEditMenu);
	
  ///////////////////////////////////////////////////////////////////
  // menuBar entry viewMenu
  pViewMenu=new QPopupMenu();
  pViewMenu->setCheckable(true);
  viewToolBar->addTo(pViewMenu);
  viewStatusBar->addTo(pViewMenu);
  viewViewBar->addTo(pViewMenu);

  ///////////////////////////////////////////////////////////////////
  // EDIT YOUR APPLICATION SPECIFIC MENUENTRIES HERE

  ///////////////////////////////////////////////////////////////////
  // menuBar entry windowMenu
  pWindowMenu = new QPopupMenu(this);
  pWindowMenu->setCheckable(true);
  connect(pWindowMenu, SIGNAL(aboutToShow()), this, SLOT(windowMenuAboutToShow()));

  ///////////////////////////////////////////////////////////////////
  // menuBar entry helpMenu
  pHelpMenu=new QPopupMenu();
  helpAboutApp->addTo(pHelpMenu);
  helpListKeyBindings->addTo(pHelpMenu);
  pHelpMenu->insertSeparator();
  pHelpMenu->insertItem(tr("What's &This"), this, SLOT(whatsThis()), SHIFT+Key_F1);


  menuBar()->insertItem(tr("&File"), pFileMenu);
  menuBar()->insertItem(tr("&Edit"), pEditMenu);
  menuBar()->insertItem(tr("&View"), pViewMenu);
  menuBar()->insertItem(tr("&Window"), pWindowMenu);
  menuBar()->insertItem(tr("&Help"), pHelpMenu);

}

void QtpLgviewerApp::initToolBar()
{
  ///////////////////////////////////////////////////////////////////
  // TOOLBAR
  fileToolbar = new QToolBar(this, "file operations");
  fileNew->addTo(fileToolbar);
  fileOpen->addTo(fileToolbar);
  fileSave->addTo(fileToolbar);
  fileToolbar->addSeparator();
  QWhatsThis::whatsThisButton(fileToolbar);

	viewtoolbar = new QToolBar(this,"view types");
	
	// view toolbar
	
	// view transformation buttons
	
	lastviewtransform = lastviewmode = 0;
	
	viewtransformbuttons = new (QToolButton *)[4];
	
	viewtransformbuttons[0] = new QToolButton(QPixmap(orthoxy_xpm),tr("orthoXY"),tr("orthogonal parallel projection to XY plane"),this,SLOT(slotSetViewTransform()),viewtoolbar);
	viewtransformbuttons[1] = new QToolButton(QPixmap(orthoyz_xpm),tr("orthoYZ"),tr("orthogonal parallel projection to YZ plane"),this,SLOT(slotSetViewTransform()),viewtoolbar);
	viewtransformbuttons[2] = new QToolButton(QPixmap(orthoxz_xpm),tr("orthoXZ"),tr("orthogonal parallel projection to XZ plane"),this,SLOT(slotSetViewTransform()),viewtoolbar);
	viewtransformbuttons[3] = new QToolButton(QPixmap(persp1_xpm),tr("persp-1"),tr("one-point-perspective projection"),this,SLOT(slotSetViewTransform()),viewtoolbar);
		
	for(int i=0; i < 4; i++) viewtransformbuttons[i]->setToggleButton(true);
	viewtransformbuttons[0]->setOn(true);
	
	viewtoolbar->addSeparator();
	
	// view mode buttons
	
	viewmodebuttons = new (QToolButton *)[4];
	
	viewmodebuttons[0] = new QToolButton(QPixmap(wireframe_xpm),tr("wireframe"),tr("display as wireframe"),this,SLOT(slotSetViewMode()),viewtoolbar);
	viewmodebuttons[1] = new QToolButton(QPixmap(backfaceculling_xpm),tr("backface culling"),tr("wireframe w.o. backfaces"),this,SLOT(slotSetViewMode()),viewtoolbar);
	viewmodebuttons[2] = new QToolButton(QPixmap(surface_xpm),tr("flat shading"),tr("flat shading"),this,SLOT(slotSetViewMode()),viewtoolbar);
	viewmodebuttons[3] = new QToolButton(QPixmap(surfaceandwf_xpm),tr("flat shading and wireframe"),tr("flat shading + wireframe"),this,SLOT(slotSetViewMode()),viewtoolbar);

	for(int i=0; i < 4; i++) viewmodebuttons[i]->setToggleButton(true);
	viewmodebuttons[0]->setOn(true);

}

void QtpLgviewerApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  //STATUSBAR
  statusBar()->message(tr("Ready."));
}

void QtpLgviewerApp::initView()
{ 
  ////////////////////////////////////////////////////////////////////
  // set the main widget here
  QVBox* view_back = new QVBox( this );
  view_back->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
  pWorkspace = new QWorkspace( view_back );
  setCentralWidget(view_back);
}


void QtpLgviewerApp::createClient(QtpLgviewerDoc* doc)
{
  QtpLgviewerView* w = new QtpLgviewerView(doc, pWorkspace,0,WDestructiveClose);
  w->installEventFilter(this);
  doc->addView(w);
  if ( pWorkspace->windowList().isEmpty() ) // show the very first window in maximized mode
    w->showMaximized();
  else
    w->show();
}

void QtpLgviewerApp::openDocumentFile(const char* file)
{
  statusBar()->message(tr("Opening file..."));
  QtpLgviewerDoc* doc;
	// check, if document already open. If yes, set the focus to the first view
  for(doc=pDocList->first(); doc > 0; doc=pDocList->next())
  {
    if(doc->pathName()==file)
    {
      QtpLgviewerView* view=doc->firstView();	
      view->setFocus();
      return;
     }
  }
  doc = new QtpLgviewerDoc();
  pDocList->append(doc);
  doc->newDocument();
  // Creates an untitled window if file is 0	
  if(!file)
  {
    untitledCount+=1;
    QString fileName=QString(tr("Untitled%1")).arg(untitledCount);
    doc->setPathName(fileName);
    doc->setTitle(fileName);
  }
  // Open the file
  else
  {
    try{doc->openDocument(file);}
    catch(QString errormsg)
    {
      QMessageBox::critical(this, tr("Error !"),tr(errormsg));
			pDocList->remove(doc);
      return;
    }
  }
  // create the window
  createClient(doc);

  statusBar()->message(tr("Ready."));
}

bool QtpLgviewerApp::queryExit()
{
  int exit=QMessageBox::information(this, tr("Quit..."),
                                    tr("Do your really want to quit?"),
                                    QMessageBox::Ok, QMessageBox::Cancel);

  if (exit==1)
  {

  }
  else
  {

  };

  return (exit==1);
}

bool QtpLgviewerApp::eventFilter(QObject* object, QEvent* event)
{
  if((event->type() == QEvent::Close)&&((QtpLgviewerApp*)object!=this))
  {
    QCloseEvent* e=(QCloseEvent*)event;
    QtpLgviewerView* pView=(QtpLgviewerView*)object;
    QtpLgviewerDoc* pDoc=pView->getDocument();
    if(pDoc->canCloseFrame(pView))
    {
      pDoc->removeView(pView);
      if(!pDoc->firstView())
        pDocList->remove(pDoc);

      e->accept();
    }
    else
      e->ignore();
  }
  return QWidget::eventFilter( object, event );    // standard event processing
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////


void QtpLgviewerApp::slotFileNew()
{
  statusBar()->message(tr("Creating new file..."));

  openDocumentFile();		

  statusBar()->message(tr("Ready."));
}

void QtpLgviewerApp::slotFileOpen()
{
  statusBar()->message(tr("Opening file..."));

  QString fileName = QFileDialog::getOpenFileName(0,"plg-Files (*.plg)",this);
  if (!fileName.isEmpty())
  {
     openDocumentFile(fileName);		
  }
  statusBar()->message(tr("Ready."));
}


void QtpLgviewerApp::slotFileSave()
{
  statusBar()->message(tr("Saving file..."));
	
  QtpLgviewerView* m = (QtpLgviewerView*)pWorkspace->activeWindow();
  if( m )
  {
    QtpLgviewerDoc* doc = m->getDocument();
    if(doc->title().contains(tr("Untitled")))
      slotFileSaveAs();
    else
      if(!doc->saveDocument(doc->pathName()))
        QMessageBox::critical (this, tr("I/O Error !"), tr("Could not save the current document !"));
  }

  statusBar()->message(tr("Ready."));
}

void QtpLgviewerApp::slotFileSaveAs()
{
  statusBar()->message(tr("Saving file under new filename..."));
  QString fn = QFileDialog::getSaveFileName(0, "plg-Files (*.plg)", this);
  if (!fn.isEmpty())
  {
    QtpLgviewerView* m = (QtpLgviewerView*)pWorkspace->activeWindow();
    if( m )
    {
      QtpLgviewerDoc* doc = m->getDocument();
      if(!doc->saveDocument(fn))
      {
         QMessageBox::critical (this, tr("I/O Error !"), tr("Could not save the current document !"));
         return;
      }
      doc->changedViewList();
    }
  }
  statusBar()->message(tr("Ready."));
}

void QtpLgviewerApp::slotFileClose()
{
  statusBar()->message(tr("Closing file..."));
	
  QtpLgviewerView* m = (QtpLgviewerView*)pWorkspace->activeWindow();
  if( m )
  {
    QtpLgviewerDoc* doc=m->getDocument();
    doc->closeDocument();
  }

  statusBar()->message(tr("Ready."));
}

void QtpLgviewerApp::slotFilePrint()
{
  statusBar()->message(tr("Printing..."));
	
  QtpLgviewerView* m = (QtpLgviewerView*) pWorkspace->activeWindow();
  if ( m )
    m->print( printer );

  statusBar()->message(tr("Ready."));
}

void QtpLgviewerApp::slotFileQuit()
{ 
  statusBar()->message(tr("Exiting application..."));
  ///////////////////////////////////////////////////////////////////
  // exits the Application
//  if(doc->isModified())
//  {
//    if(queryExit())
//    {
//      qApp->quit();
//    }
//    else
//    {
//
//    };
//  }
//  else
//  {
    qApp->quit();
//  };

  statusBar()->message(tr("Ready."));
}

void QtpLgviewerApp::slotEditUndo()
{
  statusBar()->message(tr("Reverting last action..."));
	
  QtpLgviewerView* m = (QtpLgviewerView*) pWorkspace->activeWindow();
  if ( m )
//   m->undo();

  statusBar()->message(tr("Ready."));
}

void QtpLgviewerApp::slotEditCut()
{
  statusBar()->message(tr("Cutting selection..."));
	
  QtpLgviewerView* m = (QtpLgviewerView*) pWorkspace->activeWindow();
  if ( m )
//  m->cut();	

  statusBar()->message(tr("Ready."));
}

void QtpLgviewerApp::slotEditCopy()
{
  statusBar()->message(tr("Copying selection to clipboard..."));

  QtpLgviewerView* m = (QtpLgviewerView*) pWorkspace->activeWindow();
  if ( m )
//  m->copy();

  statusBar()->message(tr("Ready."));
}

void QtpLgviewerApp::slotEditPaste()
{
  statusBar()->message(tr("Inserting clipboard contents..."));
	
  QtpLgviewerView* m = (QtpLgviewerView*) pWorkspace->activeWindow();
  if ( m )
//   m->paste();

  statusBar()->message(tr("Ready."));
}


void QtpLgviewerApp::slotViewToolBar(bool toggle)
{
  statusBar()->message(tr("Toggle toolbar..."));
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
   if (toggle== false)
  {
    fileToolbar->hide();
  }
  else
  {
    fileToolbar->show();
  };

 statusBar()->message(tr("Ready."));
}

void QtpLgviewerApp::slotViewStatusBar(bool toggle)
{
  statusBar()->message(tr("Toggle statusbar..."));
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  
  if (toggle == false)
  {
    statusBar()->hide();
  }
  else
  {
    statusBar()->show();
  }

  statusBar()->message(tr("Ready."));
}

void QtpLgviewerApp::slotWindowNewWindow()
{
  statusBar()->message(tr("Opening new document view..."));
	
  QtpLgviewerView* m = (QtpLgviewerView*) pWorkspace->activeWindow();
  if ( m ){
    QtpLgviewerDoc* doc = m->getDocument();
    createClient(doc);
  }

  statusBar()->message(tr("Ready."));
}

void QtpLgviewerApp::slotHelpAbout()
{
  QMessageBox::about(this,tr("About..."),
                     tr("QtpLgviewer\nVersion " VERSION
                     "\n(c) 2001 by Rdiger Knrig"));
}

void QtpLgviewerApp::slotStatusHelpMsg(const QString &text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message of whole statusbar temporary (text, msec)
  statusBar()->message(text, 2000);
}

void QtpLgviewerApp::windowMenuAboutToShow()
{
  pWindowMenu->clear();	
	windowNewWindow->addTo(pWindowMenu);
	windowCascade->addTo(pWindowMenu);
	windowTile->addTo(pWindowMenu);
	
  if ( pWorkspace->windowList().isEmpty() )
  {
    windowAction->setEnabled(false);
  }
  else
  {
    windowAction->setEnabled(true);
  }

  pWindowMenu->insertSeparator();

  QWidgetList windows = pWorkspace->windowList();
  for ( int i = 0; i < int(windows.count()); ++i )
  {
    int id = pWindowMenu->insertItem(QString("&%1 ").arg(i+1)+windows.at(i)->caption(), this, SLOT( windowMenuActivated( int ) ) );
    pWindowMenu->setItemParameter( id, i );
    pWindowMenu->setItemChecked( id, pWorkspace->activeWindow() == windows.at(i) );
  }
}

void QtpLgviewerApp::windowMenuActivated( int id )
{
  QWidget* w = pWorkspace->windowList().at( id );
  if ( w )
    w->setFocus();
}

/** set view transform of the actual view */
void QtpLgviewerApp::slotSetViewTransform()
{
	int i;
	
	viewtransformbuttons[lastviewtransform]->setOn(false);
	
	for(i = 0; i < 4; i++)
		if(viewtransformbuttons[i]->isOn()) break;
	
	i = (i < 4) ? i : lastviewtransform;
	viewtransformbuttons[i]->setOn(true);
	
	if(i != lastviewtransform)
	{
		QtpLgviewerView* m = (QtpLgviewerView*) pWorkspace->activeWindow();
  	if ( m )
  		{
  			m->slotSetViewTransform((MyPoly3D::tmatrix::viewtransfrom_t) i);
  		}
  }
	
	lastviewtransform = i;
	
}
/** set view mode of curret display */
void QtpLgviewerApp::slotSetViewMode()
{
	int i;
	QtpLgviewerView::view_flags flags = QtpLgviewerView::none;
	
	// ensure that only one button is pressed
	viewmodebuttons[lastviewmode]->setOn(false);
	
	for(i = 0; i < 4; i++)
		if(viewmodebuttons[i]->isOn()) break;
	
	i = (i < 4) ? i : lastviewmode;
	viewmodebuttons[i]->setOn(true);
	
	// normal wireframe?
	if(viewmodebuttons[0]->isOn()) flags = QtpLgviewerView::wireframe;
	// wireframe and backface culling?
	if(viewmodebuttons[1]->isOn()) flags = QtpLgviewerView::wireframe | QtpLgviewerView::backfaceculling;
	// simple flat shading ?
	if(viewmodebuttons[2]->isOn()) flags = QtpLgviewerView::flatshading | QtpLgviewerView::backfaceculling;
	//flat shading plus wireframe plus backfaceculling?
	if(viewmodebuttons[3]->isOn()) flags = QtpLgviewerView::flatshading | QtpLgviewerView::backfaceculling | QtpLgviewerView::wireframe;
	
	
	if(i != lastviewmode)
	{
		QtpLgviewerView* m = (QtpLgviewerView*) pWorkspace->activeWindow();
  	if ( m )
  		{
  			m->slotSetViewType(flags);
  		}
  }
	
	lastviewmode = i;
}

/** shows an information dialog box
	* showing the key bindings for the view
	*/
void QtpLgviewerApp::slotHelpKeys()
{
	QMessageBox::information(this,"key bindings","Navigation keys:\n"
	" cursor keys \t\t move camera left/right/forward/backward\n"
	" cursor keys + shift \t let the camera look left/right/up/down\n"
	" cursor keys + alt \t move camera left/right/up/down\n"
	" plus/minus \t\t zoom in/out view\n"
	" enter \t\t set zoom to 1:1.");
}

/**
	* invokes a dialog for specifying
	* geometric transformation applied to the
	* whole scene */
	
void QtpLgviewerApp::slotEditTransformations()
{
	QtpLgviewerView* m = (QtpLgviewerView*) pWorkspace->activeWindow();
  if ( m )
  {
		MygeomTransformDialog dlg(this);
		
		if(dlg.exec() == QDialog::Accepted)
		{
			// get document of active view
			QtpLgviewerDoc *doc = m->getDocument();

			MyPoly3D::tmatrix transfm = dlg.getTransform();
	
			// lock access to documents data
			while(!doc->accessmutex.lock());
			// apply transformation matrix
			for(vector<MyPoly3D::point4f>::iterator point = doc->pointlist.begin(); point != doc->pointlist.end(); point++)
			{
				// transformate points
				*point =  *point * transfm;
			}
			
			doc->accessmutex.unlock();
			doc->setModified(true);
			doc->updateAllViews(0);
		}
	}
}

/** enables / disables View Transformation bar */
void QtpLgviewerApp::slotViewViewTransformBar(bool toggle)
{
  statusBar()->message(tr("Toggle view-bar..."));

  if (toggle == false)
  {
    viewtoolbar->hide();
  }
  else
  {
    viewtoolbar->show();
  }

  statusBar()->message(tr("Ready."));
}

/**
	* updates view specific toolbars if
	* a new view becomes focus
	*/
void QtpLgviewerApp::slotUpdateViewToolbars()
{
	QtpLgviewerView* m = (QtpLgviewerView*) pWorkspace->activeWindow();
  if ( m )
  	{
			int viewtransform = (int) m->getViewTransform();
			int viewmode = 0;
			QtpLgviewerView::view_flags viewtype = m->getViewFlags();
			
			/** deactivate current selected view transform button */
			viewtransformbuttons[lastviewtransform]->setOn(false);
			/** activate new button */
			viewtransformbuttons[viewtransform]->setOn(true);
			/** update lastviewtransform */
			lastviewtransform = viewtransform;
			
			/** deactivate current selected view mode button */
			viewmodebuttons[lastviewmode]->setOn(false);	
			/** decode view mode button index from flags */
	
			if(!(viewtype ^ (QtpLgviewerView::wireframe | QtpLgviewerView::backfaceculling))) viewmode = 1;
			if(!(viewtype ^  (QtpLgviewerView::flatshading | QtpLgviewerView::backfaceculling))) viewmode = 2;
			if(!(viewtype ^ (QtpLgviewerView::flatshading | QtpLgviewerView::backfaceculling | QtpLgviewerView::wireframe))) viewmode = 3;
	
			viewmodebuttons[viewmode]->setOn(true);
			lastviewmode = viewmode;
		}
}


/** invoked to set up the current view */
void QtpLgviewerApp::slotEditViewSettings()
{
	QtpLgviewerView* m = (QtpLgviewerView*) pWorkspace->activeWindow();
  if ( m )
  	{
  		MyViewPropertyClass dlg;
  		// configure dialog
  		float bw = m->getFocalLength();
  		dlg.slotSetFocalLength(bw);
  		dlg.slotSetBGColor(m->getBackgroundColor());
  		
  		if(dlg.exec() == QDialog::Accepted)
  		{
  			float newbw = dlg.getFocalLength();
  			if(fabs(bw-newbw) > 5e-2)
  			{
  				m->slotSetFocalLength(newbw);	
  				m->slotSetViewTransform(m->getViewTransform());
  			}
  			m->slotSetBackgroundColor(dlg.getBGColor());
  		}
  	}
}
